@@ -51,6 +51,8 @@
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include "nautilus-file-changes-queue.h"
 #include "nautilus-file-private.h"
 #include "nautilus-desktop-icon-file.h"
@@ -1890,7 +1892,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 
 	TransferInfo *transfer_info;
 	SyncTransferInfo *sync_transfer_info;
-	GnomeVFSVolume *volume;
 	GnomeVFSResult result;
 	gboolean target_is_trash;
 	gboolean duplicate;
@@ -1915,6 +1916,12 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	if (target_dir != NULL) {
 		if (eel_uri_is_trash (target_dir)) {
 			target_is_trash = TRUE;
+		} else if (eel_uri_is_desktop (target_dir)) {
+			char *desktop_dir_uri;
+
+			desktop_dir_uri = nautilus_get_desktop_directory_uri ();
+			target_dir_uri = gnome_vfs_uri_new (desktop_dir_uri);
+			g_free (desktop_dir_uri);
 		} else {
 			target_dir_uri = gnome_vfs_uri_new (target_dir);
 		}
@@ -1957,11 +1964,29 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			have_nonmapping_source = TRUE;
 		}
 
-		volume = nautilus_get_enclosing_volume (source_uri);
-		if (volume != NULL && gnome_vfs_volume_is_read_only (volume)) {
-			have_readonly_source = TRUE;
+		if (!have_readonly_source) {
+			GnomeVFSVolume *volume;
+			char *text_uri, *path;
+
+			text_uri = gnome_vfs_uri_to_string (source_uri, GNOME_VFS_URI_HIDE_NONE);
+
+			path = gnome_vfs_get_local_path_from_uri (text_uri);
+
+			volume = NULL;
+			if (path != NULL) {
+				/* TODO should we resolve symlinks ourselves, or should
+				 * gnome_vfs_volume_monitor_get_volume_for_path use lstat? */
+				volume = gnome_vfs_volume_monitor_get_volume_for_path (gnome_vfs_get_volume_monitor (), path);
+			}
+
+			if (volume != NULL && gnome_vfs_volume_is_read_only (volume)) {
+				have_readonly_source = TRUE;
+			}
+
+			gnome_vfs_volume_unref (volume);
+			g_free (path);
+			g_free (text_uri);
 		}
-		gnome_vfs_volume_unref (volume);
 
 		/* Note: this could be null if we're e.g. copying the top level file of a web site */
 		source_dir_uri = gnome_vfs_uri_get_parent (source_uri);
@@ -2349,10 +2374,20 @@ nautilus_file_operations_new_folder (GtkWidget *parent_view,
 	NewFolderTransferState *state;
 	SyncTransferInfo *sync_transfer_info;
 
-	/* pass in the target directory and the new folder name as a destination URI */
-	parent_uri = gnome_vfs_uri_new (parent_dir);
-	/* localizers: the initial name of a new folder  */
+	g_assert (parent_dir != NULL);
 
+	/* pass in the target directory and the new folder name as a destination URI */
+	if (eel_uri_is_desktop (parent_dir)) {
+		char *desktop_dir_uri;
+
+		desktop_dir_uri = nautilus_get_desktop_directory_uri ();
+		parent_uri = gnome_vfs_uri_new (desktop_dir_uri);
+		g_free (desktop_dir_uri);
+	} else {
+		parent_uri = gnome_vfs_uri_new (parent_dir);
+	}
+
+	/* localizers: the initial name of a new folder  */
 	dirname = g_filename_from_utf8 (_("untitled folder"), -1, NULL, NULL, NULL);
 	uri = gnome_vfs_uri_append_file_name (parent_uri, dirname);
 	g_free (dirname);
@@ -2540,7 +2575,21 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 	g_assert (template_uri != NULL);
 
 	/* pass in the target directory and the new folder name as a destination URI */
-	parent_uri = gnome_vfs_uri_new (parent_dir);
+	if (eel_uri_is_desktop (parent_dir)) {
+		tmp = nautilus_get_desktop_directory_uri ();
+		parent_uri = gnome_vfs_uri_new (tmp);
+		g_free (tmp);
+	} else if (eel_uri_is_trash (parent_dir) ||
+		   eel_uri_is_search (parent_dir)) {
+		parent_uri = NULL;
+	} else {
+		parent_uri = gnome_vfs_uri_new (parent_dir);
+	}
+
+	if (parent_uri == NULL) {
+		(*done_callback) (NULL, data);
+		return;
+	}
 
 	source_uri = gnome_vfs_uri_new (template_uri);
 	if (source_uri == NULL) {
@@ -2849,6 +2898,239 @@ nautilus_file_operations_empty_trash (GtkWidget *parent_view)
 	}
 }
 
+struct RecursivePermissionsInfo {
+	GnomeVFSAsyncHandle *handle;
+	GnomeVFSURI *current_dir;
+	GnomeVFSURI *current_file;
+	GList *files;
+	GList *directories;
+	GnomeVFSFilePermissions file_permissions;
+	GnomeVFSFilePermissions file_mask;
+	GnomeVFSFilePermissions dir_permissions;
+	GnomeVFSFilePermissions dir_mask;
+	NautilusSetPermissionsCallback callback;
+	gpointer callback_data;
+};
+
+struct FileInfo {
+	char *name;
+	GnomeVFSFilePermissions permissions;
+};
+
+struct DirInfo {
+	GnomeVFSURI *uri;
+	GnomeVFSFilePermissions permissions;
+};
+
+static void set_permissions_run (struct RecursivePermissionsInfo *info);
+
+static void
+set_permissions_set_file_info (GnomeVFSAsyncHandle *handle,
+			       GnomeVFSResult result,
+			       GnomeVFSFileInfo *old_file_info,
+			       gpointer callback_data)
+{
+	struct RecursivePermissionsInfo *info;
+	GnomeVFSFileInfo *vfs_info;
+	GnomeVFSURI *uri;
+	char *uri_str;
+	struct FileInfo *file_info;
+	
+	info = callback_data;
+
+	if (result == GNOME_VFS_OK && info->current_file != NULL) {
+		uri_str = gnome_vfs_uri_to_string (info->current_file, GNOME_VFS_URI_HIDE_NONE);
+		nautilus_file_changes_queue_file_changed (uri_str);
+		g_free (uri_str);
+	}
+
+	if (info->current_file) {
+		gnome_vfs_uri_unref (info->current_file);
+	}
+	if (info->files == NULL) {
+		/* No more files, process more dirs */
+		set_permissions_run (info);
+		return;
+	}
+
+	file_info = info->files->data;
+	info->files = g_list_delete_link (info->files, info->files);
+
+	uri = gnome_vfs_uri_append_file_name (info->current_dir,
+					      file_info->name);
+	info->current_file = uri;
+	
+	vfs_info = gnome_vfs_file_info_new ();
+	vfs_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS;
+	vfs_info->permissions = 
+			(file_info->permissions & ~info->file_mask) |
+			info->file_permissions;
+	
+	gnome_vfs_async_set_file_info (&info->handle, uri, vfs_info,
+				       GNOME_VFS_SET_FILE_INFO_PERMISSIONS,
+				       GNOME_VFS_FILE_INFO_DEFAULT,
+				       GNOME_VFS_PRIORITY_DEFAULT,
+				       set_permissions_set_file_info,
+				       info);
+	
+	gnome_vfs_file_info_unref (vfs_info);
+	g_free (file_info->name);
+	g_free (file_info);
+	
+}
+
+static void
+set_permissions_got_files (GnomeVFSAsyncHandle *handle,
+			  GnomeVFSResult result,
+			  GList *list,
+			  guint entries_read,
+			  gpointer callback_data)
+{
+	struct RecursivePermissionsInfo *info;
+	GnomeVFSFileInfo *vfs_info;
+	GList *l;
+	
+	info = callback_data;
+
+	if (result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF) {
+		for (l = list; l != NULL; l = l->next) {
+			vfs_info = l->data;
+
+			if (strcmp (vfs_info->name, ".") == 0 ||
+			    strcmp (vfs_info->name, "..") == 0 ||
+			    !(vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS)) {
+				continue;
+			}
+
+			if (vfs_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
+				struct DirInfo *dir_info;
+
+				dir_info = g_new (struct DirInfo, 1);
+				dir_info->uri = gnome_vfs_uri_append_file_name (info->current_dir,
+										vfs_info->name);
+				dir_info->permissions = vfs_info->permissions;
+				info->directories = g_list_prepend (info->directories,
+								    dir_info);
+			} else {
+				struct FileInfo *file_info;
+				file_info = g_new (struct FileInfo, 1);
+				file_info->name = g_strdup (vfs_info->name);
+				file_info->permissions = vfs_info->permissions;
+				info->files = g_list_prepend (info->files, file_info);
+			}
+		}
+	}
+
+
+	if (result != GNOME_VFS_OK) {
+		/* Finished with this dir, work on the files */
+		info->current_file = NULL;
+		set_permissions_set_file_info (NULL, GNOME_VFS_OK, NULL, info);
+	}
+	
+}
+
+/* Also called for the toplevel dir */
+static void
+set_permissions_load_dir (GnomeVFSAsyncHandle *handle,
+			  GnomeVFSResult result,
+			  GnomeVFSFileInfo *file_info,
+			  gpointer callback_data)
+{
+	struct RecursivePermissionsInfo *info;
+	char *uri_str;
+
+	info = callback_data;
+	
+	if (result == GNOME_VFS_OK && handle != NULL) {
+		uri_str = gnome_vfs_uri_to_string (info->current_dir, GNOME_VFS_URI_HIDE_NONE);
+		nautilus_file_changes_queue_file_changed (uri_str);
+		g_free (uri_str);
+	}
+	
+	gnome_vfs_async_load_directory_uri (&info->handle,
+					    info->current_dir,
+					    GNOME_VFS_FILE_INFO_DEFAULT,
+					    50,
+					    GNOME_VFS_PRIORITY_DEFAULT,
+					    set_permissions_got_files,
+					    info);
+}
+
+static void
+set_permissions_run (struct RecursivePermissionsInfo *info)
+{
+	struct DirInfo *dir_info;
+	GnomeVFSFileInfo *vfs_info;
+	
+	gnome_vfs_uri_unref (info->current_dir);
+
+	if (info->directories == NULL) {
+		/* No more directories, finished! */
+		info->callback (info->callback_data);
+		/* All parts of info should be freed now */
+		g_free (info);
+		return;
+	}
+
+	dir_info = info->directories->data;
+	info->directories = g_list_delete_link (info->directories, info->directories);
+
+	info->current_dir = dir_info->uri;
+
+	vfs_info = gnome_vfs_file_info_new ();
+	vfs_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS;
+	vfs_info->permissions = 
+			(dir_info->permissions & ~info->dir_mask) |
+			info->dir_permissions;
+
+	gnome_vfs_async_set_file_info (&info->handle,
+				       info->current_dir,
+				       vfs_info,
+				       GNOME_VFS_SET_FILE_INFO_PERMISSIONS,
+				       GNOME_VFS_FILE_INFO_DEFAULT,
+				       GNOME_VFS_PRIORITY_DEFAULT,
+				       set_permissions_load_dir,
+				       info);
+
+	gnome_vfs_file_info_unref (vfs_info);
+	g_free (dir_info);
+}
+	
+void
+nautilus_file_set_permissions_recursive (const char                     *directory,
+					 GnomeVFSFilePermissions         file_permissions,
+					 GnomeVFSFilePermissions         file_mask,
+					 GnomeVFSFilePermissions         dir_permissions,
+					 GnomeVFSFilePermissions         dir_mask,
+					 NautilusSetPermissionsCallback  callback,
+					 gpointer                        callback_data)
+{
+	struct RecursivePermissionsInfo *info;
+
+	info = g_new (struct RecursivePermissionsInfo, 1);
+
+	info->files = NULL;
+	info->directories = NULL;
+	info->file_permissions = file_permissions;
+	info->file_mask = file_mask;
+	info->dir_permissions = dir_permissions;
+	info->dir_mask = dir_mask;
+	info->callback = callback;
+	info->callback_data = callback_data;
+
+	info->current_dir = gnome_vfs_uri_new (directory);
+
+	if (info->current_dir == NULL) {
+		info->callback (info->callback_data);
+		g_free (info);
+		return;
+	}
+	
+	set_permissions_load_dir (NULL, GNOME_VFS_OK, NULL, info);
+}
+
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
