@@ -47,7 +47,7 @@ typedef struct {
 } NautilusHorizontalSplitterClass;
 
 /* NautilusHorizontalSplitter public methods */
-GtkType    nautilus_horizontal_splitter_get_type (void);
+GType      nautilus_horizontal_splitter_get_type (void);
 GtkWidget *nautilus_horizontal_splitter_new      (void);
 
 gboolean   nautilus_horizontal_splitter_is_hidden	(NautilusHorizontalSplitter *splitter);
@@ -56,6 +56,8 @@ void	   nautilus_horizontal_splitter_hide		(NautilusHorizontalSplitter *splitter
 void	   nautilus_horizontal_splitter_show		(NautilusHorizontalSplitter *splitter);
 void	   nautilus_horizontal_splitter_expand		(NautilusHorizontalSplitter *splitter);
 void	   nautilus_horizontal_splitter_toggle_position	(NautilusHorizontalSplitter *splitter);
+void	   nautilus_horizontal_splitter_pack2           (NautilusHorizontalSplitter *splitter,
+							 GtkWidget                  *child2);
 
 G_END_DECLS
 
