@@ -45,8 +45,7 @@ typedef struct {
 	double x, y;
 	
 	/* Scale factor (stretches icon). */
-	/* ALEX TODO: only allow once scale */
-	double scale_x, scale_y;
+	double scale;
 
 	/* Whether this item is selected. */
 	eel_boolean_bit is_selected : 1;
@@ -54,6 +53,9 @@ typedef struct {
 	/* Whether this item was selected before rubberbanding. */
 	eel_boolean_bit was_selected_before_rubberband : 1;
 
+	/* Whether this item is visible in the view. */
+	eel_boolean_bit is_visible : 1;
+
 	/* Whether a monitor was set on this icon. */
 	eel_boolean_bit is_monitored : 1;
 
@@ -155,6 +157,7 @@ struct NautilusIconContainerDetails {
 	DragState drag_state;
 	gboolean drag_started;
 	StretchState stretch_start;
+	gboolean drag_allow_moves;
 
 	gboolean icon_selected_on_button_down;
 	NautilusIcon *double_click_icon[2]; /* Both clicks in a double click need to be on the same icon */
@@ -267,8 +270,7 @@ void          nautilus_icon_container_move_icon                   (NautilusIconC
 								   NautilusIcon          *icon,
 								   int                    x,
 								   int                    y,
-								   double                 scale_x,
-								   double                 scale_y,
+								   double                 scale,
 								   gboolean               raise,
 								   gboolean               snap,
 								   gboolean		  update_position);
