@@ -35,7 +35,6 @@
 #include "nautilus-application.h"
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-main.h"
-#include "nautilus-signaller.h"
 #include "nautilus-location-bar.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-query-editor.h"
@@ -62,7 +61,7 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkliststore.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
@@ -85,6 +84,7 @@
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-search-directory.h>
+#include <libnautilus-private/nautilus-signaller.h>
 #include <math.h>
 #include <sys/time.h>
 
@@ -95,10 +95,6 @@
  */
 #include "nautilus-desktop-window.h"
 
-/* FIXME bugzilla.gnome.org 41245: hardwired sizes */
-#define SIDE_PANE_MINIMUM_WIDTH 1
-#define SIDE_PANE_MINIMUM_HEIGHT 400
-
 #define MAX_TITLE_LENGTH 180
 
 #define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/MenuBar/Other Menus/Bookmarks/Bookmarks Placeholder"
@@ -115,7 +111,7 @@ enum {
 	ARG_APP
 };
 
-static int side_pane_width_auto_value = SIDE_PANE_MINIMUM_WIDTH;
+static int side_pane_width_auto_value = 0;
 
 static void add_sidebar_panels                       (NautilusNavigationWindow *window);
 static void load_view_as_menu                        (NautilusWindow           *window);
@@ -143,6 +139,48 @@ static void nautilus_navigation_window_show_location_bar_temporarily (NautilusNa
 GNOME_CLASS_BOILERPLATE (NautilusNavigationWindow, nautilus_navigation_window,
 			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
 
+static void
+location_button_toggled_cb (GtkToggleButton *toggle,
+			    NautilusNavigationWindow *window)
+{
+	gboolean is_active;
+
+	is_active = gtk_toggle_button_get_active (toggle);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY, is_active);
+
+	if (is_active) {
+		nautilus_navigation_bar_activate (NAUTILUS_NAVIGATION_BAR (window->navigation_bar));
+	}
+}
+
+static gboolean
+location_button_should_be_active (NautilusNavigationWindow *window)
+{
+	return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+}
+
+static GtkWidget *
+location_button_create (NautilusNavigationWindow *window)
+{
+	GtkWidget *image;
+	GtkWidget *button;
+
+	image = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+
+	button = g_object_new (GTK_TYPE_TOGGLE_BUTTON,
+			       "image", image,
+			       "active", location_button_should_be_active (window),
+			       NULL);
+
+	gtk_tooltips_set_tip (window->details->tooltips, button,
+			      _("Toggle between button and text-based location bar"), NULL);
+
+	g_signal_connect (button, "toggled",
+			  G_CALLBACK (location_button_toggled_cb), window);
+	return button;
+}
+
 static void
 nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 {
@@ -155,6 +193,10 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	
 	window->details = g_new0 (NautilusNavigationWindowDetails, 1);
 
+	window->details->tooltips = gtk_tooltips_new ();
+	g_object_ref (window->details->tooltips);
+	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
+
 	window->details->content_paned = nautilus_horizontal_splitter_new ();
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  window->details->content_paned,
@@ -210,7 +252,7 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	location_bar = gtk_toolbar_new ();
 	window->details->location_bar = location_bar;
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_widget_show (hbox);
 
 	item = gtk_tool_item_new ();
@@ -221,6 +263,10 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	gtk_toolbar_insert (GTK_TOOLBAR (location_bar),
 			    item, -1);
 
+	window->details->location_button = location_button_create (window);
+	gtk_box_pack_start (GTK_BOX (hbox), window->details->location_button, FALSE, FALSE, 0);
+	gtk_widget_show (window->details->location_button);
+
 	window->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
  	gtk_widget_show (window->path_bar);
 	
@@ -309,14 +355,25 @@ static void
 always_use_location_entry_changed (gpointer callback_data)
 {
 	NautilusNavigationWindow *window;
+	gboolean use_entry;
 
 	window = NAUTILUS_NAVIGATION_WINDOW (callback_data);
 
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+	use_entry = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+
+	if (use_entry) {
 		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
 	} else {
 		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
 	}
+
+	g_signal_handlers_block_by_func (window->details->location_button,
+					 G_CALLBACK (location_button_toggled_cb),
+					 window);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->details->location_button), use_entry);
+	g_signal_handlers_unblock_by_func (window->details->location_button,
+					   G_CALLBACK (location_button_toggled_cb),
+					   window);
 }
 
 static int
@@ -434,7 +491,7 @@ side_pane_size_allocate_callback (GtkWidget *widget,
 		if (eel_preferences_key_is_writable (NAUTILUS_PREFERENCES_SIDEBAR_WIDTH)) {
 			eel_preferences_set_integer
 				(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, 
-				 allocation->width);
+				 allocation->width <= 1 ? 0 : allocation->width);
 		}
 	}
 }
@@ -564,6 +621,22 @@ nautilus_navigation_window_unrealize (GtkWidget *widget)
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
+static gboolean
+nautilus_navigation_window_state_event (GtkWidget *widget,
+					GdkEventWindowState *event)
+{
+	if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED) {
+		eel_preferences_set_boolean (NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED,
+					     event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
+	}
+
+	if (GTK_WIDGET_CLASS (parent_class)->window_state_event != NULL) {
+		return GTK_WIDGET_CLASS (parent_class)->window_state_event (widget, event);
+	}
+
+	return FALSE;
+}
+
 static void
 nautilus_navigation_window_destroy (GtkObject *object)
 {
@@ -581,6 +654,11 @@ nautilus_navigation_window_destroy (GtkObject *object)
 	window->zoom_control = NULL;
 
 	window->details->content_paned = NULL;
+
+	if (window->details->tooltips) {
+		g_object_unref (window->details->tooltips);
+		window->details->tooltips = NULL;
+	}
 	
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
@@ -814,21 +892,26 @@ real_load_view_as_menu (NautilusWindow *window)
 	load_view_as_menu (window);
 }
 
-static void
+static gboolean
 real_set_title (NautilusWindow *window, const char *title)
 {
 	char *full_title;
 	char *window_title;
-	
-	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS,
-			 set_title, (window, title));
-	
-	full_title = g_strdup_printf (_("%s - File Browser"), title);
+	gboolean changed;
 
-	window_title = eel_str_middle_truncate (full_title, MAX_TITLE_LENGTH);
-	gtk_window_set_title (GTK_WINDOW (window), window_title);
-	g_free (window_title);
-	g_free (full_title);
+	changed = EEL_CALL_PARENT_WITH_RETURN_VALUE
+		(NAUTILUS_WINDOW_CLASS, set_title, (window, title));
+
+	if (changed) {
+		full_title = g_strdup_printf (_("%s - File Browser"), title);
+
+		window_title = eel_str_middle_truncate (full_title, MAX_TITLE_LENGTH);
+		gtk_window_set_title (GTK_WINDOW (window), window_title);
+		g_free (window_title);
+		g_free (full_title);
+	}
+
+	return changed;
 }
 
 static char *
@@ -1417,8 +1500,7 @@ nautilus_navigation_window_save_geometry (NautilusNavigationWindow *window)
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
-	if (GTK_WIDGET(window)->window &&
-	    !(gdk_window_get_state (GTK_WIDGET(window)->window) & GDK_WINDOW_STATE_MAXIMIZED)) {
+	if (GTK_WIDGET(window)->window) {
 		geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 		
 		if (eel_preferences_key_is_writable (NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY)) {
@@ -1427,6 +1509,12 @@ nautilus_navigation_window_save_geometry (NautilusNavigationWindow *window)
 				 geometry_string);
 		}
 		g_free (geometry_string);
+
+		if (eel_preferences_key_is_writable (NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED)) {
+			eel_preferences_set_boolean
+				(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED, 
+				 gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_MAXIMIZED);
+		}
 	}
 }
 
@@ -1461,6 +1549,7 @@ nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_navigation_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_navigation_window_show;
 	GTK_WIDGET_CLASS (class)->unrealize = nautilus_navigation_window_unrealize;
+	GTK_WIDGET_CLASS (class)->window_state_event = nautilus_navigation_window_state_event;
 	NAUTILUS_WINDOW_CLASS (class)->load_view_as_menu = real_load_view_as_menu;
 	NAUTILUS_WINDOW_CLASS (class)->set_content_view_widget = real_set_content_view_widget;
 	NAUTILUS_WINDOW_CLASS (class)->set_throbber_active = real_set_throbber_active;
